(function() {
	'use strict';

	angular.module('ipconfigure.orchid.directive').directive('motionReductionSupport', ['$timeout', motionReductionSupport]);

	function motionReductionSupport() {
		return {
			restrict: 'E',
			scope: {
				streamId: '=',
				driver: '=',
				encoder: '=',
				frameRate: '=',
				capabilities: '=',
				govLength: '=',
				recordingStyle: '='
			},
			template: ' ' +
				'<div>' +
				'<div class="row" ng-show="recordingStyle ===\'Recording only when motion detected\'">' +
				'	<div class="form-group col-md-8 text-justify text-danger">' +
				'		<em>' +
				'			Attention! With this mode, motion detection accuracy may vary and result in video loss.' +
				'		</em>' +
				'	</div>' +
				'</div>' +
				'<div ng-show="recordingStyle === \'Continuous recording with motion adaptive frame rate\'">' +
				'	<div class="row" ng-show="driver === \'ONVIF\' && encoder === \'H264\'">' +
				'       <div class="form-group col-md-4">' +
				'          <label for="Frame Rate During Motion">Frame Rate During Motion</label>' +
				'          <input id="Frame Rate During Motion" class="form-control" ng-model="frameRate">' +
				'          <input type="range" min="{{minFrameRate}}" max="{{maxFrameRate}}" ng-model="frameRate" step="1">' +
				'       </div>' +
				'       <div class="form-group col-md-4">' +
				'          <label for="Frame Rate During Non-Motion">Frame Rate During Non-Motion</label>' +
				'          <input id="Frame Rate During Non-Motion" class="form-control" type="text" value="{{nonMotionFrameRateValues[nonMotionFrameRateIndex]|number:2}}" disabled>' +
				'          <input type="range" min="0" max="{{nonMotionFrameRateValues.length - 1}}" ng-model="nonMotionFrameRateIndex" step="1">' +
				'       </div>' +
				'	</div>' +
				'	<div class="row" ng-show="driver === \'ONVIF\' && encoder !== \'H264\'">' +
				'		<div class="form-group col-md-8 text-justify text-danger">' +
				'			<em>' +
				'				Attention! The Motion Reduction feature currently works only with an H264 encoded stream, so to take advantage of this feature, you\'ll need to configure your Encoder below to use H264.' +
				'			</em>' +
				'		</div>' +
				'	</div>' +
				'	<div class="row" ng-show="driver === \'Generic RTSP\'">' +
				'		<div class="form-group col-md-8 text-justify">' +
				'			<em>' +
				'				Please note that our Motion Reduction feature requires an H264 stream with a specific Frame Rate and GOV Length.' +
				'				Since you\'re using our Generic RTSP driver, we cannot control the encoder, frame rate, or GOV Length, so to take full advantage of Motion Reduction, you\'ll need to configure the source stream appropriately.' +
				'			</em>' +
				'		</div>' +
				'	</div>' +
				'</div>' +
				'</div>',
			link: function(scope) {
				var loaded = false;

				scope.nonMotionFrameRateValues = [1, 2, 3];
				scope.nonMotionFrameRateIndex = 0;

				// the following series of $watch statements are critical in initially loading data and manipulating it real time.  the order and
				// redundancy in which they are placed is paramount to how this directive works due to the overlapping nature of the properties below.

				// we first watch streamId to monitor when the user selects a stream
				scope.$watch('streamId', function() {
					loaded = false;
				});

				// frameRate is watched next so we can generate unique gov length options for the desired frame rate
				scope.$watch('frameRate', function() {
					var i;

					if (angular.isUndefined(scope.frameRate) === false) {
						scope.nonMotionFrameRateIndex = 0;
						scope.nonMotionFrameRateValues = [];

						// the frame rate has changed, so we need to re-generate the available option non-motion frame rate options
						for (i = scope.frameRate; i >= 1; i -= 1) {
							scope.nonMotionFrameRateValues.push(scope.frameRate / i);
						}

						if (scope.nonMotionFrameRateValues.length === 0) {
							scope.nonMotionFrameRateValues = [1];
						}

						if (loaded === true) {
							// default the gov length to something usable
							scope.govLength = parseInt(scope.frameRate / scope.nonMotionFrameRateValues[scope.nonMotionFrameRateIndex], 10);
						}
					}
				});

				// we have to watch the non-motion frame rate index so that we can select the appropriate gov length to achieve the desired effect for the user
				scope.$watch('nonMotionFrameRateIndex', function() {
					if (loaded === true && scope.recordingStyle === 'Continuous recording with motion adaptive frame rate') {
						scope.govLength = parseInt(scope.frameRate / scope.nonMotionFrameRateValues[scope.nonMotionFrameRateIndex], 10);
					}
				});

				// finally we watch streamId again so that we can initially load the correct non-motion frame rate index
				scope.$watch('streamId', function() {
					// stream id has changed, set the nonMotionFrameRateIndex based on the frame rate and gov length, which should be watched above this
					scope.nonMotionFrameRateIndex = scope.nonMotionFrameRateValues.indexOf(scope.frameRate / scope.govLength);

					// if we calculated an impossible value, provide a default
					if (scope.nonMotionFrameRateIndex === -1) {
						scope.nonMotionFrameRateIndex = 0;

						if (scope.recordingStyle === 'Continuous recording with motion adaptive frame rate') {
							// if mr reducer is selected, baseline the gov length to the ui isn't ever lying
							scope.govLength = parseInt(scope.frameRate / scope.nonMotionFrameRateValues[scope.nonMotionFrameRateIndex], 10);
						}
					}

					// default min/max frame rate in case we can't find one
					scope.minFrameRate = 1;
					scope.maxFrameRate = 30;

					// set frame rate according to capabilities
					if (scope.capabilities && scope.capabilities.stream && scope.capabilities.stream.Video && scope.capabilities.stream.Video.Encoder && scope.capabilities.stream.Video.Encoder.capabilities && scope.capabilities.stream.Video.Encoder.capabilities.H264 && scope.capabilities.stream.Video.Encoder.capabilities.H264.FrameRate && scope.capabilities.stream.Video.Encoder.capabilities.H264.FrameRate) {
						scope.minFrameRate = scope.capabilities.stream.Video.Encoder.capabilities.H264.FrameRate.min;
						scope.maxFrameRate = scope.capabilities.stream.Video.Encoder.capabilities.H264.FrameRate.max;
					}

					loaded = true;
				});

				scope.$watch('encoder', function() {
					_createMessages();
					_showHideCorrectBits();
				});

				scope.$watch('recordingStyle', function() {
					_createMessages();

					if (scope.recordingStyle === 'Continuous recording with motion adaptive frame rate') {
						// normalize the gov length so if the drop down is changed, but no sliders are changed, things get saved correctly
						scope.govLength = parseInt(scope.frameRate / scope.nonMotionFrameRateValues[scope.nonMotionFrameRateIndex], 10);
					}

					_showHideCorrectBits();
				});

				function _createMessages() {
					var frameRateInput = document.querySelector('#FrameRate'),
						govLengthInput = document.querySelector('#GOVLength'),
						message = 'These values can be manipulated above while Motion Reduction is selected.',
						div;

					// check to see if it's been created, create only if it hasn't
					if (frameRateInput !== null && govLengthInput !== null && frameRateInput.parentNode.querySelector('.mr-reducer-notice') === null) {
						div = document.createElement('div');
						div.setAttribute('class', 'mr-reducer-notice');
						div.textContent = message;
						frameRateInput.parentNode.appendChild(div);

						div = document.createElement('div');
						div.setAttribute('class', 'mr-reducer-notice');
						div.textContent = message;
						govLengthInput.parentNode.appendChild(div);
					}
				}

				function _showHideCorrectBits() {
					var frameRateInput = document.querySelector('#FrameRate'),
						govLengthInput = document.querySelector('#GOVLength'),
						frameRateDiv,
						govLengthDiv;

					// update the ui
					if (frameRateInput !== null && frameRateInput.parentNode.querySelector('.mr-reducer-notice') !== null) {
						frameRateDiv = frameRateInput.parentNode.querySelector('div.mr-reducer-notice');

						if (govLengthInput !== null) {
							govLengthDiv = govLengthInput.parentNode.querySelector('div.mr-reducer-notice');
						}

						if (scope.recordingStyle === 'Continuous recording with motion adaptive frame rate') {
							frameRateInput.style.display = 'none';
							frameRateInput.nextElementSibling.style.display = 'none';
							frameRateDiv.style.display = 'block';

							if (govLengthInput !== null) {
								govLengthInput.style.display = 'none';
								govLengthInput.nextElementSibling.style.display = 'none';
								govLengthDiv.style.display = 'block';
							}
						}
						else {
							frameRateDiv.style.display = '';
							frameRateInput.style.display = '';
							frameRateInput.nextElementSibling.style.display = '';

							if (govLengthInput !== null) {
								govLengthDiv.style.display = '';
								govLengthInput.style.display = '';
								govLengthInput.nextElementSibling.style.display = '';
							}
						}
					}
				}
			}
		};
	}
}());
