(function($console) {
	'use strict';

	window.ipConfigure = window.ipConfigure || {};
	ipConfigure.components = ipConfigure.components || {};

	/**
	 * Compatability-mode video class, injects an IMG into the element indicated by the id parameter.
	 * Exposes events and allows control of the "video".
	 *
	 * @param {string} id
	 * @param {object} options backgroundColor (html color), debug (bool), requestBaseUrl (callback)
	 *
	 * @namespace ipConfigure.components.video
	 */
	ipConfigure.components.videoCompatibility = function(id, options) {
		var that = this,
			_pluginContainer = document.querySelector('#' + id),
			_imgContainer,
			_img,
			_state = 'ready',
			_sized = false,
			_sizingInfo = {},
			_clockTime,
			_currentTime,
			_isBuffering = false,
			_srcTimer,
			_resizeTimer,
			_tickTimer,
			_renegotiateSizerTimer,
			_cachedBaseUrl = '',
			_cachedFps = 0,
			_defaultOptions = {
				backgroundColor: '#EEEEEE',
				debug: false,
				getTime: function() {
					_error('getTime event not subscribed to');
					return 0;
				},
				requestBaseUrl: function() {
					_error('requestBaseUrl event not subscribed to');
					return 'http://notfound';
				},
				onAudioKnown: function(hasAudio) {
					_log('onAudioKnown, hasAudio: ', hasAudio);
				},
				onEndReached: function() {
				},
				onPlayFromTime: function() {
				},
				onPlayLive: function() {
				},
				onStateChange: function(s) {
					_log('onStateChange event not subscribed to: ', s);
				},
				onError: '',
				stream: undefined
			},
			_options = ipConfigure.core.mergeOptions(_defaultOptions, options);

		// private functions

		function _error() {
			$console.error.apply($console, arguments);
		}

		function _log() {
			if (_options.debug === true) {
				$console.log.apply($console, arguments);
			}
		}

		function _createComponent() {
			var html = '<div class="plugin compatibility" style="background-color: ' + _options.backgroundColor + '">' +
				'   <img onError="' + _options.onError + '">' +
				'   <div class="plugin-mask"></div>' +
				'</div>';

			_pluginContainer.className += ' ipc-video';
			_pluginContainer.innerHTML = html;

			_imgContainer = _pluginContainer.querySelector('.plugin');
			_img = _pluginContainer.querySelector('img');
		}

		function _setSize(skipRenegotiation) {
			var containerAspectRatio,
				favor;

			// cancel previous renegotiate attempt
			if (_renegotiateSizerTimer) {
				clearTimeout(_renegotiateSizerTimer);
			}

			if (_sized === true) {
				containerAspectRatio = _imgContainer.clientWidth / _imgContainer.clientHeight;

				favor = _sizingInfo.aspectRatio > containerAspectRatio ? 'width' : 'height';

				// figure out if we need to favor width or height
				_img.style[favor === 'width' ? 'height' : 'width'] = '';
				_img.style[favor] = '100%';

				// renegotiate resize size 5 seconds after this function has run
				if (skipRenegotiation !== true && that.isPlaying === true) {
					_renegotiateSizerTimer = setTimeout(function() {
						_negotiateBaseUrl(that.isLive === true ? 0 : _options.getTime(), 'resize');
					}, 2000);
				}
			}
		}

		function _onImgLoad() {
			var minWait = 33,
				wait = minWait;

			if (_isBuffering === true) {
				_isBuffering = false;

				_state = 'playing';
				_options.onStateChange('playing');
			}

			if (_sized === false) {
				_options.stream.aspectRatio = [_img.clientWidth, _img.clientHeight];
				_sized = true;
				_sizingInfo.aspectRatio = _img.naturalWidth / _img.naturalHeight;
				_setSize(true);
			}

			_imgContainer.style.opacity = 1;

			// delay based on FPS, sometimes 0 is provided... 1000 / 0 is Infinity... so don't let that happen
			if (_cachedFps === 0) {
				wait = 1000;
			}
			else if (_cachedFps > 0) {
				wait = Math.max(minWait, (1000 / _cachedFps));
			}

			_setSrcDelayed(wait);
		}

		function _onImgLoadError() {
			_log('MediaPlayerEncounteredError');

			_state = 'ready';
			_options.onStateChange('error', 'Unable to display video at this time');
		}

		function _setSrc() {
			if (_cachedBaseUrl !== '') {
				_img.src = _cachedBaseUrl + '?' + Date.now();
			}
		}

		function _setSrcDelayed(delayBetweenFrames) {
			_srcTimer = setTimeout(function() {
				_currentTime += Date.now() - _clockTime;
				_clockTime = Date.now();

				if (that.isPlaying === true) {
					_setSrc();
				}
			}, delayBetweenFrames);
		}

		function _stop() {
			if (_state !== 'ready') {
				_cachedBaseUrl = '';

				clearTimeout(_srcTimer);

				_state = 'ready';
				_options.onStateChange('ready');
			}
		}

		function _negotiateBaseUrl(startTime, via) {
			_log('NEGOTIATE NOW', startTime);

			_state = 'loading';
			_options.onStateChange('loading');

			_options.requestBaseUrl({
				start: startTime,
				width: _imgContainer.parentElement.parentElement.clientWidth,
				height: _imgContainer.parentElement.parentElement.clientHeight,
				via: via
			}, function(baseUrl, fpsOrReason) {
				var notification;

				if (that.isPlaying === true) {
					if (baseUrl === false) {
						_cachedBaseUrl = '';

						if (fpsOrReason && fpsOrReason.indexOf('has been reached') > -1) {
							notification = fpsOrReason;
						}
						else {
							notification = 'Unable to display video at this time';
						}

						_state = 'ready';
						_options.onStateChange('error', notification);
					}
					else if (baseUrl === '') {
						_isBuffering = true;

						_cachedBaseUrl = baseUrl;
						_cachedFps = fpsOrReason;
					}
					else {
						_cachedBaseUrl = baseUrl;
						_cachedFps = fpsOrReason;
						_setSrc();
					}
				}
			});
		}

		function _init() {
			_log('video options', _options);

			// create the img tag and overlays
			_createComponent();

			// create subscriptions to necessary events
			_img.addEventListener('load', _onImgLoad);
			_img.addEventListener('error', _onImgLoadError);

			// monitor for changes in size, so we can request an appropriately sized image from the server
			_resizeTimer = setInterval(function tick() {
				var firstTime = angular.isUndefined(_sizingInfo.containerWidth),
					width = _imgContainer.parentElement.parentElement.clientWidth,
					height = _imgContainer.parentElement.parentElement.clientHeight;

				if (_sizingInfo.containerWidth !== width || _sizingInfo.containerHeight !== height) {
					_sizingInfo.containerWidth = width;
					_sizingInfo.containerHeight = height;

					_log('SET SIZE FIRST TIME', firstTime);
					_setSize(firstTime);
				}
			}, 333);

			_tickTimer = setInterval(function tick() {
				_currentTime += Date.now() - _clockTime;
				_clockTime = Date.now();
			}, 1000);
		}

		// public properties

		that.isLive = false;

		that.isPlaying = false;

		that.options = _options;

		// public methods

		that.getVolume = function() {
			return 0;
		};

		that.setVolume = function() {
			// placebo
		};

		that.getCurrentTime = function() {
			return _currentTime;
		};

		that.getVideo = function() {
			return _img;
		};

		/**
		 * @param {number} startTime number of milliseconds since 1970
		 * @param {boolean} autoPause automatically pauses the video when it begins playing when true
		 */
		that.playFromTime = function(startTime, autoPause) {
			if (typeof _options.stream === 'undefined') {
				_error('videoCompatibility.options.stream not set');
			}
			else {
				// reset locals in case the video dimensions have changed
				_sized = false;

				// set the loading state and stop whatever is playing
				_clockTime = Date.now();
				_currentTime = startTime;
				_isBuffering = true;
				_stop();

				// set necessary properties
				that.isLive = false;
				that.isPlaying = autoPause ? false : true;

				// let em know
				_options.onPlayFromTime();

				// start pulling frames
				_negotiateBaseUrl(startTime, 'play');
			}
		};

		that.playLive = function() {
			if (typeof _options.stream === 'undefined') {
				_error('videoCompatibility.options.stream not set');
			}
			else {
				// reset locals in case the video dimensions have changed
				_sized = false;

				// set necessary properties
				that.isLive = true;
				that.isPlaying = true;

				// set the loading state and stop whatever is playing
				_clockTime = Date.now();
				_currentTime = _clockTime;
				_isBuffering = true;
				_stop();

				// let em know
				_options.onPlayLive();

				// start pulling frames
				_negotiateBaseUrl(0, 'play');
			}
		};

		that.stop = function() {
			// clean up public properties
			that.isLive = false;
			that.isPlaying = false;

			_stop();
		};

		that.destroy = function() {
			_log('video destroy');

			that.isPlaying = false;

			if (_srcTimer) {
				clearTimeout(_srcTimer);
			}

			if (_renegotiateSizerTimer) {
				clearTimeout(_renegotiateSizerTimer);
			}

			clearInterval(_resizeTimer);
			clearInterval(_tickTimer);

			_img.removeEventListener('load', _onImgLoad, false);
			_img.removeEventListener('error', _onImgLoadError, false);
		};

		// initialize

		_init();
	};
}(console));
