(function() {
	'use strict';

	angular.module('ipconfigure.orchid.service').factory('LowBandwidthStreamService', ['$http', '$log', '$timeout', 'AuthService', LowBandwidthStreamService]);

	function LowBandwidthStreamService($http, $log, $timeout, AuthService) {
		var _queue = [],
			_stagingQueue = [],
			_listeningTimer,
			lastRefresh = 0;

		function listenTick() {
			var baseUrl,
				pendingStreamsExist = false;

			while (_stagingQueue.length > 0) {
				_queue.push(_stagingQueue[0]);
				_stagingQueue.splice(0, 1);
			}

			if (_queue.length > 0) {
				// TODO this will not support mesh
				baseUrl = _queue[0].stream.cameraExpanded.instance.baseUrl;

				// see if there are any pending streams
				_queue.some(function(item) {
					if (item.status === 'pending') {
						pendingStreamsExist = true;
						return true;
					}
				});

				// poll the server if there are pending streams or 10 seconds has elapsed since our last refresh
				if (pendingStreamsExist === true || (Date.now() - lastRefresh) > 3000) {
					lastRefresh = Date.now();

					// poll /service/low-bandwidth/streams/ while waiting for streams in our queue to go status==='ready'
					$http.get(baseUrl + '/service/low-bandwidth/streams/')
						.success(function(response) {
							//console.log(new Date(), 'q', _queue.map(function(item) { return item.payload.streamId + ' ' + item.payload.id + ' ' + item.payload.state.status; }));
							//console.log(new Date(), 's', Array.isArray(response.streams) ? response.streams.map(function(s) { return s.streamId + ' ' + s.id + ' ' + s.state.status; }) : 'null');

							// iterate through _queue
							_queue.forEach(function(item) {
								var found = false;

								if (Array.isArray(response.streams) === true) {
									response.streams.forEach(function(stream) {
										if (item.payload.id === stream.id) {
											found = true;

											item.payload = stream;
											item.status = stream.state.status;

											// if we have a stream in our list that's now ready
											if (stream.state.status === 'ready') {
												$log.debug('stream ready:', stream.id);

												item.callback(true, stream);
											}
											else if (stream.state.status === 'failed') {
												//console.log(new Date(), 'failed', stream.streamId, stream.id);

												$log.debug('stream failed:' + stream.state.status, stream.id);

												item.callback(false, stream);
											}
										}
									});
								}

								// if we have a stream that's not in the list, callback(false); removeFromQueue();
								if (found === false) {
									$log.error('stream not found on server:', item.payload.id);

									item.callback(false, item.payload);
								}
							});

							// poll again later
							_listeningTimer = $timeout(listenTick, 250, false);
						})
						.error(function(data, status, headers, config) {
							if (status !== 0) {
								$log.error('request to GET /service/low-bandwidth/streams failed', data, status, config);
							}

							// poll again later if we haven't been logged out
							if (!data || (data && data.code !== 403)) {
								_listeningTimer = $timeout(listenTick, 250, false);
							}
							else {
								// the user isn't authenticated anymore, reinitialize everything in case they sign back in
								_queue = [];
								_stagingQueue = [];
								_listeningTimer = undefined;
								lastRefresh = 0;
							}
						});
				}
				else {
					// if there's nothing to do, just check back later
					_listeningTimer = $timeout(listenTick, 250, false);
				}
			}
			else {
				// if there's nothing in the queue, just check back later
				_listeningTimer = $timeout(listenTick, 250, false);
			}
		}

		function listen() {
			if (angular.isUndefined(_listeningTimer) === true) {
				// kick off the listen timer the first time this service is actually used, wait at least
				// 100ms to give the service a chance to spin up a stream for us
				_listeningTimer = $timeout(listenTick, 100, false);
			}
		}

		function createStream(stream, options, callback) {
			var payload = {
				streamId: stream.id,
				resolution: {
					width: options.width,
					height: options.height
				},
				startTime: options.start,
				status: 'pending',
				sync: true
			};

			if (payload.startTime > 0) {
				// take into account server time difference from client time
				payload.startTime -= (window.clientServerTime && typeof window.clientServerTime.diff === 'number' ? window.clientServerTime.diff : 0);
			}

			//console.log(new Date(), 'create', payload.streamId);

			$http.post(stream.cameraExpanded.instance.baseUrl + '/service/low-bandwidth/streams', payload)
				.success(function(response) {
					//console.log(new Date(), 'created', payload.streamId, response.id);

					// let the callback know that we've created a stream so it can cancel it later if it needs to
					callback(true, response);

					// enqueue this into our staging queue while we wait for it to be ready
					// we're using a staging queue here for race conditions (if our $http.post here starts and
					// finishes while we're waiting for a response from $http.get above, we'll be working from
					// a stale list and assume things have been deleted out from under us on the server.
					_stagingQueue.push({ status: 'pending', stream: stream, callback: callback, payload: response });

					listen();
				})
				.error(function(data, status, headers, config) {
					$log.error('post failed for /service/low-bandwidth/streams', data, status, headers, config);

					callback(false, {});
				});
		}

		function destroyStream(lowBandwidthModeStreamResource) {
			// check the queue and remove this one if it's in there
			_queue.some(function(item, index) {
				if (item.payload.id === lowBandwidthModeStreamResource.id) {
					//console.log(new Date(), 'remove', lowBandwidthModeStreamResource.streamId, lowBandwidthModeStreamResource.id, '(' + index + ')');
					_queue.splice(index, 1);

					return true;
				}
			});

			//console.log(new Date(), 'destroy', lowBandwidthModeStreamResource.streamId, lowBandwidthModeStreamResource.id);

			if (AuthService.getAuthenticatedUser() !== false) {
				$http.delete(lowBandwidthModeStreamResource.href)
					.error(function(data, status, headers, config) {
						$log.error('delete failed for ' + lowBandwidthModeStreamResource.href, data, status, headers, config);
					});
			}
		}

		return {
			createStream: createStream,
			destroyStream: destroyStream
		};
	}
}());
