(function() {
	'use strict';

	angular.module('ipconfigure.orchid.service').factory('MotionDetectionService', ['$http', '$log', 'PlayerComponentRegistrar', 'TimeDeltaService', MotionDetectionService]);

	function MotionDetectionService($http, $log, PlayerComponentRegistrar, TimeDeltaService) {
		var _lastTimelinePosition = {
			master: {
				start: false,
				stop: false
			}
		};

		function getMinSegment(timeline) {
			return timeline.getTimeLengthFromPixels(1 / 3);
		}

		function setMotionDetectionAreas(masterTimeline, totallyCompleteCallback) {
			var linkedComponents = {},
				linkedCount = 0,
				unlinkedComponents = {},
				unlinkedCount = 0,
				components = PlayerComponentRegistrar.getComponents(),
				linkedBaseUrl,
				unlinkedBaseUrl,
				streamIds = '',
				diff,
				fineTime,
				start,
				stop,
				url;

			function _createHttpGetCallback(success, providedComponents) {
				if (success === true) {
					return function(response) {
						if (response.events !== null) {
							providedComponents.timeline.setMotionEvents(response.events);
						}

						unlinkedCount -= 1;
						_totallyCompleteCheck();
					};
				}
				else {
					return function(data, status, headers, config) {
						if (status !== 0) {
							$log.error(data, status, headers, config);
						}

						unlinkedCount -= 1;
						_totallyCompleteCheck();
					};
				}
			}

			function _handleUnlinkedComponent(providedComponents) {
				diff = TimeDeltaService.getDifference();
				fineTime = providedComponents.timeline.getFineTime();
				start = fineTime[0] - diff;
				stop = fineTime[1] - diff;
				url = providedComponents.playerScope.instance.baseUrl + '/service/events/camera-stream/histogram?eventType=IPC_MOTION_EVENT&start=' + start + '&stop=' + stop + '&id=' + providedComponents.playerScope.stream.id + '&minSegment=' + getMinSegment(providedComponents.timeline);

				if (!_lastTimelinePosition[unlinkedBaseUrl] || (_lastTimelinePosition[unlinkedBaseUrl].start !== start && _lastTimelinePosition[unlinkedBaseUrl].stop !== stop)) {
					// keep track of the last time so we don't unnecessarily spam the server
					_lastTimelinePosition[unlinkedBaseUrl] = {
						start: start,
						stop: stop
					};

					$http.get(url)
						.success(_createHttpGetCallback(true, providedComponents))
						.error(_createHttpGetCallback(false, providedComponents));
				}
				else {
					$log.debug('skip unlinked because it\'s the same');
					unlinkedCount -= 1;
					_totallyCompleteCheck();
				}
			}

			function _extractStreamIdsFromComponents(specificComponents) {
				return specificComponents.reduce(function(value, providedComponents) {
					return value + ',' + providedComponents.playerScope.stream.id;
				}, '').substring(1);
			}

			function _totallyCompleteCheck() {
				$log.debug('_totallyCompleteCheck', linkedCount, unlinkedCount);

				if (linkedCount === 0 && unlinkedCount === 0) {
					totallyCompleteCallback();
				}
			}

			if (components.length === 0) {
				masterTimeline.setMotionEvents([]);
				totallyCompleteCallback(0);
			}
			else {
				// build a collection of instances and components that we need to request motion data for
				components.forEach(function(component) {
					if (component.playerScope.linked === true) {
						// create an array if necessary
						if (Array.isArray(linkedComponents[component.playerScope.instance.baseUrl]) === false) {
							linkedComponents[component.playerScope.instance.baseUrl] = [];
						}

						// add this component for later processing
						linkedComponents[component.playerScope.instance.baseUrl].push(component);

						linkedCount += 1;
					}
					else {
						// create an array if necessary
						if (Array.isArray(unlinkedComponents[component.playerScope.instance.baseUrl]) === false) {
							unlinkedComponents[component.playerScope.instance.baseUrl] = [];
						}

						// add this component for later processing
						unlinkedComponents[component.playerScope.instance.baseUrl].push(component);

						unlinkedCount += 1;
					}
				});

				$log.debug('unlinkedComponents', unlinkedCount, unlinkedComponents);
				$log.debug('linkedComponents', linkedCount, linkedComponents);

				// process motion events for unlinked components
				if (unlinkedCount > 0) {
					for (unlinkedBaseUrl in unlinkedComponents) {
						if (unlinkedComponents.hasOwnProperty(unlinkedBaseUrl) === true) {
							unlinkedComponents[unlinkedBaseUrl].forEach(_handleUnlinkedComponent);
						}
					}
				}

				if (linkedCount > 0) {
					diff = TimeDeltaService.getDifference();
					fineTime = masterTimeline.getFineTime();
					start = fineTime[0] - diff;
					stop = fineTime[1] - diff;

					// the upper bound shouldn't exceed the current time
					if (stop > Date.now() - diff) {
						stop = Date.now() - diff;
					}

					// if the time has changed significantly, pull motion data
					if (_lastTimelinePosition.master && (_lastTimelinePosition.master.start !== start || _lastTimelinePosition.master.stop !== stop)) {
						for (linkedBaseUrl in linkedComponents) {
							if (linkedComponents.hasOwnProperty(linkedBaseUrl) === true) {
								streamIds = _extractStreamIdsFromComponents(linkedComponents[linkedBaseUrl]);
							}
						}

						// TODO need to support multiple instances
						url = '/service/events/camera-stream/histogram?eventType=IPC_MOTION_EVENT&start=' + start + '&stop=' + stop + '&id=' + streamIds + '&minSegment=' + getMinSegment(masterTimeline);

						// keep track of the last time so we don't unnecessarily spam the server
						_lastTimelinePosition.master.start = start;
						_lastTimelinePosition.master.stop = stop;

						$http.get(url)
							.success(function(response) {
								// TODO need to support multiple instances
								if (response.events === null) {
									masterTimeline.setMotionEvents([]);
								}
								else {
									masterTimeline.setMotionEvents(response.events);
								}

								linkedCount = 0;
								_totallyCompleteCheck();
							})
							.error(function() {
								// TODO need to support multiple instances
								linkedCount = 0;
								_totallyCompleteCheck();
							});
					}
					else {
						$log.debug('skip linked because it\'s the same');
						linkedCount = 0;
						_totallyCompleteCheck();
					}
				}
				else {
					masterTimeline.setMotionEvents([]);
				}
			}
		}

		function fetchMotionEvent(direction, time, callback) {
			var components = PlayerComponentRegistrar.getComponents(),
				streamIds,
				url,
				timeDiff = TimeDeltaService.getDifference(),
				serverTime = time - timeDiff;

			// grab linked stream ids stream ids
			streamIds = components.reduce(function(ids, component) {
				return ids + ((component.playerScope.linked === true) ? ',' + component.playerScope.stream.id : '');
			}, '').substring(1);

			if (streamIds !== '') {
				url = '/service/events/camera-stream?eventType=IPC_MOTION_EVENT&start=' + serverTime + '&id=' + streamIds + '&count=' + ((direction === 'prev') ? -5 : 5);

				$http.get(url).success(function(response) {
					var jumpTo,
						jumpDiff,
						requiredDiff,
						i;

					if (response.events !== null) {
						// Offset by one second so that the start of the motion event will be visible after stream has finished loading.
						jumpDiff = 1000 - timeDiff;

						jumpTo = response.events[0].start - jumpDiff;

						// find an event at least 5 seconds prior to the one we're on or 1 second after (to account for that one second offset we added in)
						requiredDiff = (direction == 'prev') ? 5000 : 1001;

						if (response.events.length > 1) {
							for (i = 0; i < response.events.length; i++) {
								jumpTo = response.events[i].start - jumpDiff;

								if (Math.abs(time - jumpTo) > requiredDiff) {
									break;
								}
							}
						}

						callback(jumpTo);
					}
					else {
						callback(false);
					}
				}, 500);
			}
			else {
				callback(false);
			}
		}

		return {
			setMotionDetectionAreas: setMotionDetectionAreas,
			fetchMotionEvent: fetchMotionEvent
		};
	}
}());
